package org.zheng.demo.aop;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;

@Aspect
@Component
public class MyAopExample {

	@After("execution(* org.zheng.demo.service.*.*(..))")
	public void afterAdvice() {
		System.out.println("AOP_afterAdvise is executed.");
	}

	@AfterReturning(pointcut = "afterReturningPC()", returning = "retV")
	public void afterReturningAdvice(String retV) {
		System.out.println("AOP_afterReturningAdvise is executed. " + retV);
	}

	@Pointcut("execution(* org.zheng.demo.service.*.*(..))")
	public void afterReturningPC() {
	}

	@AfterThrowing(pointcut = "matchingAll()", throwing = "e")
	public void afterThrowingAdvice(RuntimeException e) {
		Thread.setDefaultUncaughtExceptionHandler((t, e1) -> System.out.println("Caught " + e1.getMessage()));
		System.out.println("AOP_afterThrowingAdvice is executed");
	}

	@Before("execution(* org.zheng.demo.service.MyService.myMethod(..))")
	public void beforeAdvice() {
		System.out.println("AOP_beforeAdvice is executed");
	}

	@Around("execution(* org.zheng.demo.service.*.*(..))")
	public Object logExecutionTime(ProceedingJoinPoint joinPoint) throws Throwable {
		long startTime = System.currentTimeMillis();

		Object proceed = joinPoint.proceed();

		long endTime = System.currentTimeMillis();
		long executionTime = endTime - startTime;

		System.out.println("AOP_aroundAdvice " + joinPoint.getSignature() + " executed in " + executionTime + "ms");
		return proceed;
	}

	@Pointcut(value = "execution(public * org.zheng..*.*(..))")
	public void matchingAll() {
	}

}
